@@ -5,6 +5,7 @@ import (
 	"fmt"
 	"log"
 	"math"
+	"math/rand"
 	"runtime"
 	"sync"
 	"time"
@@ -61,6 +62,36 @@ type serviceWithName struct {
 	name    string
 }
 
+// Jitter returns the sum of the input duration and a random jitter.  It is
+// compatible with the jitter functions in github.com/lthibault/jitterbug.
+type Jitter interface {
+	Jitter(time.Duration) time.Duration
+}
+
+// NoJitter does not apply any jitter to the input duration
+type NoJitter struct{}
+
+// Jitter leaves the input duration d unchanged.
+func (NoJitter) Jitter(d time.Duration) time.Duration { return d }
+
+// DefaultJitter is the jitter function that is applied when spec.BackoffJitter
+// is set to nil.
+type DefaultJitter struct {
+	rand *rand.Rand
+}
+
+// Jitter will jitter the backoff time by uniformly distributing it into
+// the range [FailureBackoff, 1.5 * FailureBackoff).
+func (dj *DefaultJitter) Jitter(d time.Duration) time.Duration {
+	// this is only called by the core supervisor loop, so it is
+	// single-thread safe.
+	if dj.rand == nil {
+		dj.rand = rand.New(rand.NewSource(time.Now().UnixNano()))
+	}
+	jitter := dj.rand.Float64() / 2
+	return d + time.Duration(float64(d)*jitter)
+}
+
 /*
 Supervisor is the core type of the module that represents a Supervisor.
 
@@ -92,11 +123,11 @@ not running, to prevent race conditions.
 */
 type Supervisor struct {
 	Name string
-	id   supervisorID
 
 	failureDecay         float64
 	failureThreshold     float64
 	failureBackoff       time.Duration
+	backoffJitter        Jitter
 	timeout              time.Duration
 	log                  func(string)
 	services             map[serviceID]serviceWithName
@@ -119,20 +150,26 @@ type Supervisor struct {
 	getAfterChan func(time.Duration) <-chan time.Time
 
 	sync.Mutex
-	state uint8
+
+	// malign leftovers
+	id            supervisorID
+	state         uint8
+	recoverPanics bool
 }
 
 // Spec is used to pass arguments to the New function to create a
 // supervisor. See the New function for full documentation.
 type Spec struct {
-	Log              func(string)
-	FailureDecay     float64
-	FailureThreshold float64
-	FailureBackoff   time.Duration
-	Timeout          time.Duration
-	LogBadStop       BadStopLogger
-	LogFailure       FailureLogger
-	LogBackoff       BackoffLogger
+	Log               func(string)
+	FailureDecay      float64
+	FailureThreshold  float64
+	FailureBackoff    time.Duration
+	BackoffJitter     Jitter
+	Timeout           time.Duration
+	LogBadStop        BadStopLogger
+	LogFailure        FailureLogger
+	LogBackoff        BackoffLogger
+	PassThroughPanics bool
 }
 
 /*
@@ -149,6 +186,7 @@ If not set, the following values are used:
  * FailureThreshold:  5 failures
  * FailureBackoff:    15 seconds
  * Timeout:           10 seconds
+ * BackoffJitter:     DefaultJitter
 
 The Log function will be called when errors occur. Suture will log the
 following:
@@ -176,6 +214,9 @@ failure count to zero.
 
 Timeout is how long Suture will wait for a service to properly terminate.
 
+The PassThroughPanics options can be set to let panics in services propagate
+and crash the program, should this be desirable.
+
 */
 func New(name string, spec Spec) (s *Supervisor) {
 	s = new(Supervisor)
@@ -209,11 +250,17 @@ func New(name string, spec Spec) (s *Supervisor) {
 	} else {
 		s.failureBackoff = spec.FailureBackoff
 	}
+	if spec.BackoffJitter == nil {
+		s.backoffJitter = &DefaultJitter{}
+	} else {
+		s.backoffJitter = spec.BackoffJitter
+	}
 	if spec.Timeout == 0 {
 		s.timeout = time.Second * 10
 	} else {
 		s.timeout = spec.Timeout
 	}
+	s.recoverPanics = !spec.PassThroughPanics
 
 	// overriding these allows for testing the threshold behavior
 	s.getNow = time.Now
@@ -441,7 +488,7 @@ func (s *Supervisor) Serve() {
 				// used only by tests
 				panic("Panicking as requested!")
 			}
-		case _ = <-s.resumeTimer:
+		case <-s.resumeTimer:
 			// We're resuming normal operation after a pause due to
 			// excessive thrashing
 			// FIXME: Ought to permit some spacing of these functions, rather
@@ -490,7 +537,7 @@ func (s *Supervisor) handleFailedService(id serviceID, err interface{}, stacktra
 		s.state = paused
 		s.Unlock()
 		s.LogBackoff(s, true)
-		s.resumeTimer = s.getAfterChan(s.failureBackoff)
+		s.resumeTimer = s.getAfterChan(s.backoffJitter.Jitter(s.failureBackoff))
 	}
 
 	s.lastFail = now
@@ -520,14 +567,16 @@ func (s *Supervisor) handleFailedService(id serviceID, err interface{}, stacktra
 
 func (s *Supervisor) runService(service Service, id serviceID) {
 	go func() {
-		defer func() {
-			if r := recover(); r != nil {
-				buf := make([]byte, 65535, 65535)
-				written := runtime.Stack(buf, false)
-				buf = buf[:written]
-				s.fail(id, r, buf)
-			}
-		}()
+		if s.recoverPanics {
+			defer func() {
+				if r := recover(); r != nil {
+					buf := make([]byte, 65535)
+					written := runtime.Stack(buf, false)
+					buf = buf[:written]
+					s.fail(id, r, buf)
+				}
+			}()
+		}
 
 		service.Serve()
 
@@ -546,10 +595,10 @@ func (s *Supervisor) removeService(id serviceID, notificationChan chan struct{},
 		delete(s.services, id)
 		s.servicesShuttingDown[id] = namedService
 		go func() {
-			successChan := make(chan bool)
+			successChan := make(chan struct{})
 			go func() {
 				namedService.Service.Stop()
-				successChan <- true
+				close(successChan)
 				if notificationChan != nil {
 					notificationChan <- struct{}{}
 				}
@@ -602,7 +651,6 @@ SHUTTING_DOWN_SERVICES:
 	}
 
 	close(s.liveness)
-	return
 }
 
 // String implements the fmt.Stringer interface.
@@ -614,7 +662,7 @@ func (s *Supervisor) sendControl(sm supervisorMessage) bool {
 	select {
 	case s.control <- sm:
 		return true
-	case _, _ = <-s.liveness:
+	case <-s.liveness:
 		return false
 	}
 }
@@ -639,7 +687,10 @@ RemoveAndWait will remove the given service from the Supervisor and attempt
 to Stop() it. It will wait up to the given timeout value for the service to
 terminate. A timeout value of 0 means to wait forever.
 
-If a nil error is returned from this function
+If a nil error is returned from this function, then the service was
+terminated normally. If either the supervisor terminates or the timeout
+passes, ErrTimeout is returned. (If this isn't even the right supervisor
+ErrWrongSupervisor is returned.)
 */
 func (s *Supervisor) RemoveAndWait(id ServiceToken, timeout time.Duration) error {
 	sID := supervisorID(id.id >> 32)
@@ -659,7 +710,7 @@ func (s *Supervisor) RemoveAndWait(id ServiceToken, timeout time.Duration) error
 
 	sentControl := s.sendControl(removeService{serviceID(id.id & 0xffffffff), notificationC})
 
-	if sentControl == false {
+	if !sentControl {
 		return ErrTimeout
 	}
 
@@ -671,7 +722,7 @@ func (s *Supervisor) RemoveAndWait(id ServiceToken, timeout time.Duration) error
 	// This occurs if the entire supervisor ends without the service
 	// having terminated, and includes the timeout the supervisor
 	// itself waited before closing the liveness channel.
-	case _, _ = <-s.liveness:
+	case <-s.liveness:
 		return ErrTimeout
 
 	// The local timeout.
